@@ -43,15 +43,13 @@
 #include <apt-pkg/aptconfiguration.h>
 
 #include <boost/format.hpp>
-#include <boost/make_shared.hpp>
-#include <boost/scoped_ptr.hpp>
-#include <boost/shared_ptr.hpp>
 #include <boost/unordered_map.hpp>
 
 #include <sigc++/bind.h>
 
 #include <cwidget/generic/util/ref_ptr.h>
 
+#include <memory>
 #include <vector>
 
 namespace cw = cwidget;
@@ -74,7 +72,6 @@ using aptitude::util::create_throttle;
 using aptitude::util::progress_info;
 using aptitude::util::throttle;
 using aptitude::views::progress;
-using boost::shared_ptr;
 
 namespace
 {
@@ -134,6 +131,15 @@ namespace
                     const cw::util::ref_ptr<m::structural_match> &match,
                     std::vector<std::string> &output)
     {
+#if APT_PKG_MAJOR >= 5
+      // with apt-1.1:
+      //
+      // - SourcePkg (and Version) are in the binary cache and available via
+      //   the VerIterator; much faster than parsing the pkgRecord
+      //
+      // - defaults to package name, no need to check if it's empty
+      output.push_back(ver.SourcePkgName());
+#else
       // I don't think FileList() *can* be invalid; this is just
       // paranoia.
       if(!ver.FileList().end())
@@ -145,6 +151,7 @@ namespace
           else
             output.push_back(srcpkg);
         }
+#endif
     }
 
     std::string format_header(const std::string &group)
@@ -161,6 +168,15 @@ namespace
                     const cw::util::ref_ptr<m::structural_match> &match,
                     std::vector<std::string> &output)
     {
+#if APT_PKG_MAJOR >= 5
+      // with apt-1.1:
+      //
+      // - SourcePkg (and Version) are in the binary cache and available via
+      //   the VerIterator; much faster than parsing the pkgRecord
+      //
+      // - defaults to package name, no need to check if it's empty
+      output.push_back(ver.SourcePkgName() + " " + ver.SourceVerStr());
+#else
       // I don't think FileList() *can* be invalid; this is just
       // paranoia.
       if(!ver.FileList().end())
@@ -184,6 +200,7 @@ namespace
 
           output.push_back(result);
         }
+#endif
     }
 
     std::string format_header(const std::string &group)
@@ -225,7 +242,7 @@ namespace
     for(std::vector<std::pair<pkgCache::VerIterator, cw::util::ref_ptr<m::structural_match> > >::const_iterator it = output.begin();
         it != output.end(); ++it)
       {
-        boost::scoped_ptr<cw::config::column_parameters> p(new search_result_column_parameters(it->second));
+        std::unique_ptr<cw::config::column_parameters> p(new search_result_column_parameters(it->second));
         pkg_ver_columnizer columnizer(it->first,
                                       show_package_names,
                                       columns,
@@ -248,9 +265,9 @@ namespace
                          group_by_option group_by,
                          show_package_names_option show_package_names,
                          bool debug,
-                         const shared_ptr<terminal_locale> &term_locale,
-                         const shared_ptr<terminal_metrics> &term_metrics,
-                         const shared_ptr<terminal_output> &term_output)
+                         const std::shared_ptr<terminal_locale> &term_locale,
+                         const std::shared_ptr<terminal_metrics> &term_metrics,
+                         const std::shared_ptr<terminal_output> &term_output)
   {
     // Set to -1 if any exact-name matches fail.  Also set to -1 if
     // there are no results at all.
@@ -259,9 +276,9 @@ namespace
     typedef std::vector<std::pair<pkgCache::VerIterator, cw::util::ref_ptr<m::structural_match> > >
       results_list;
 
-    const shared_ptr<progress> search_progress_display =
+    const std::shared_ptr<progress> search_progress_display =
       create_progress_display(term_locale, term_metrics, term_output);
-    const shared_ptr<throttle> search_progress_throttle =
+    const std::shared_ptr<throttle> search_progress_throttle =
       create_throttle();
 
     results_list output;
@@ -269,7 +286,7 @@ namespace
     for(std::vector<cw::util::ref_ptr<m::pattern> >::const_iterator pIt = patterns.begin();
         pIt != patterns.end(); ++pIt)
       {
-        const shared_ptr<progress> search_progress =
+        const std::shared_ptr<progress> search_progress =
           create_search_progress(serialize_pattern(*pIt),
                                  search_progress_display,
                                  search_progress_throttle);
@@ -405,7 +422,7 @@ namespace
 
     if(group_by_policy != NULL)
       {
-        typedef boost::unordered_map<std::string, boost::shared_ptr<results_list> >
+        typedef boost::unordered_map<std::string, std::shared_ptr<results_list> >
           results_by_group_map;
 
         results_by_group_map by_groups;
@@ -433,7 +450,7 @@ namespace
                  results_by_group_map::iterator found = by_groups.find(group);
                  if(found == by_groups.end())
                    {
-                     boost::shared_ptr<results_list> cell = boost::make_shared<results_list>();
+                     std::shared_ptr<results_list> cell = std::make_shared<results_list>();
 
                      by_groups[group] = cell;
                      cell->push_back(*results_it);
@@ -444,7 +461,7 @@ namespace
            }
         }
 
-        typedef std::vector<std::pair<std::string, boost::shared_ptr<results_list> > >
+        typedef std::vector<std::pair<std::string, std::shared_ptr<results_list> > >
           results_by_group_list;
 
         results_by_group_list by_groups_list(by_groups.begin(), by_groups.end());
@@ -535,7 +552,7 @@ int cmdline_versions(int argc, char *argv[], const char *status_fname,
                      group_by_option group_by,
                      show_package_names_option show_package_names)
 {
-  shared_ptr<terminal_io> term = create_terminal();
+  std::shared_ptr<terminal_io> term = create_terminal();
 
   int real_width=-1;
 
@@ -559,6 +576,12 @@ int cmdline_versions(int argc, char *argv[], const char *status_fname,
       real_width = tmp;
     }
 
+  if(!disable_columns && !pkg_item::pkg_columnizer::check_valid_display_format(display_format, PACKAGE "::CmdLine::Version-Display-Format" " (or -F)"))
+    {
+      _error->DumpErrors();
+      return -1;
+    }
+
   std::wstring wdisplay_format;
 
   if(!cw::util::transcode(display_format.c_str(), wdisplay_format))
@@ -568,7 +591,7 @@ int cmdline_versions(int argc, char *argv[], const char *status_fname,
       return -1;
     }
 
-  boost::scoped_ptr<cw::config::column_definition_list> columns;
+  std::unique_ptr<cw::config::column_definition_list> columns;
   columns.reset(parse_columns(wdisplay_format,
                               pkg_item::pkg_columnizer::parse_column_type,
                               pkg_item::pkg_columnizer::defaults));
