@@ -1,6 +1,7 @@
 // apt.h  -*-c++-*-
 //
 //  Copyright 1999-2002, 2004-2005, 2007-2010 Daniel Burrows
+//  Copyright 2015 Manuel A. Fernandez Montecelo
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -26,8 +27,7 @@
 
 #include <string.h>
 
-#include <boost/shared_ptr.hpp>
-
+#include <memory>
 #include <utility>
 #include <vector>
 
@@ -52,7 +52,6 @@ namespace aptitude
 class OpProgress;
 class pkgRecords;
 class pkgSourceList;
-class pkg_hier;
 class signalling_config;
 class undo_list;
 class resolver_manager;
@@ -64,15 +63,6 @@ extern resolver_manager *resman;
 extern pkgSourceList *apt_source_list;
 extern pkgRecords *apt_package_records;
 
-pkg_hier *get_user_pkg_hier();
-// Currently, package hierarchies are (by default) loaded using
-// DATADIR/aptitude/hier_groups and ~/.aptitude/pkgclass.  This is to
-// facilitate the editing of hierarchies.  user_pkg_hier contains the
-// information which is loaded in this way.  It is accessible only
-// through an accessor method in order to implement lazy loading (so
-// people who don't use the browser don't take the hit of having to
-// load in the (BIG) hierarchy info file)
-
 void apt_preinit(const char *rootdir);
 // Performs initialization of stuff that has to happen before apt_init is
 // called (eg, pkgInitialize and setting up the undo structure)
@@ -133,10 +123,6 @@ extern sigc::signal0<void> cache_closed;
 // This means that anyone using it should immediately drop references to it.
 // Generally followed by cache_reloaded; always followed by cache_reloaded
 // or cache_reload_failed.
-extern sigc::signal0<void> hier_reloaded;
-// Announces that user_pkg_hier has been reloaded.  (it's necessary to
-// do this BEFORE cache_reloaded, as some things try to use the old
-// user_pkg_hier in cache_reloaded if we don't)
 extern sigc::signal0<void> cache_reloaded;
 // Announces that the cache has been reloaded
 extern sigc::signal0<void> cache_reload_failed;
@@ -152,7 +138,7 @@ extern sigc::signal0<void> consume_errors;
 /** \brief Used to cache downloaded data, to avoid multiple
  *  downloads of items such as changelogs and screenshots.
  */
-extern boost::shared_ptr<aptitude::util::file_cache> download_cache;
+extern std::shared_ptr<aptitude::util::file_cache> download_cache;
 
 void apt_dumpcfg(const char *root);
 // Dumps a subtree of the configuration to ~/.aptitude/config
@@ -280,6 +266,57 @@ pkgCache::VerIterator install_version(const pkgCache::PkgIterator &pkg,
 pkgCache::DepIterator is_conflicted(const pkgCache::VerIterator &ver,
 				    aptitudeDepCache &cache);
 
+/** Check if the auto-installed package can be removed, or if installed or
+ * to-be-installed packages depend on it.
+ *
+ * @param pkg The package to check
+ *
+ * @cache Dependency cache
+ *
+ * @param follow_recommends Whether to follow recommends
+ *
+ * @param follow_suggests Whether to follow suggests
+ *
+ * @return Whether the action is allowed or not.
+ */
+bool can_remove_autoinstalled(const pkgCache::PkgIterator& pkg,
+			      aptitudeDepCache& cache,
+			      bool follow_recommends,
+			      bool follow_suggests);
+
+/** Check if package is virtual
+ *
+ * @param pkg The package to check
+ *
+ * @return Whether the package is virtual or not
+ */
+inline bool is_virtual(const pkgCache::PkgIterator& pkg)
+{
+  return (pkg.VersionList().end() && ! pkg.ProvidesList().end());
+}
+
+/** Check if package is installed
+ *
+ * @param pkg The package to check
+ *
+ * @return Whether the package is installed or not
+ */
+inline bool is_installed(const pkgCache::PkgIterator& pkg)
+{
+  return (pkg->CurrentState != pkgCache::State::NotInstalled && pkg->CurrentState != pkgCache::State::ConfigFiles);
+}
+
+/** Check if package is auto installed
+ *
+ * @param state State of the package to check
+ *
+ * @return Whether the package is auto installed or not
+ */
+inline bool is_auto_installed(const pkgDepCache::StateCache& state)
+{
+  return (state.Flags & pkgCache::Flag::Auto);
+}
+
 /** A pair (veriterator,verfile) -- used for building a list of
  *  versions sorted by file location.
  */
