@@ -1,6 +1,7 @@
 // aptcache.cc
 //
 //  Copyright 1999-2009, 2011 Daniel Burrows
+//  Copyright 2015 Manuel A. Fernandez Montecelo
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -26,6 +27,7 @@
 #include "aptitude_resolver_universe.h"
 #include "aptitudepolicy.h"
 #include "config_signal.h"
+#include "dpkg_selections.h"
 #include <generic/apt/matching/match.h>
 #include <generic/apt/matching/parse.h>
 #include <generic/apt/matching/pattern.h>
@@ -197,7 +199,7 @@ aptitudeDepCache::aptitudeDepCache(pkgCache *Cache, Policy *Plcy)
 {
   // When the "install recommended packages" flag changes, collect garbage.
 #if 0
-  aptcfg->connect("Apt::Install-Recommends",
+  aptcfg->connect("APT::Install-Recommends",
 		  sigc::bind(sigc::mem_fun(*this,
 					   &pkgDepCache::MarkAndSweep),
 			     (undo_group *) NULL));
@@ -353,6 +355,7 @@ bool aptitudeDepCache::build_selection_list(OpProgress &Prog, bool WithLock,
       package_states[i].reinstall=false;
       package_states[i].user_tags.clear();
       package_states[i].remove_reason=manual;
+      package_states[i].original_selection_state = pkgCache::State::Unknown;
       package_states[i].selection_state = pkgCache::State::Unknown;
       package_states[i].previously_auto_package = false;
     }
@@ -458,6 +461,7 @@ bool aptitudeDepCache::build_selection_list(OpProgress &Prog, bool WithLock,
 	      candver=section.FindS("Version");
 
 	      pkg_state.selection_state=(pkgCache::State::PkgSelectedState) section.FindI("State", pkgCache::State::Unknown);
+	      pkg_state.original_selection_state = static_cast<pkgCache::State::PkgSelectedState>(pkg->SelectedState);
 	      pkgCache::State::PkgSelectedState last_dselect_state
 		= (pkgCache::State::PkgSelectedState)
 		    section.FindI("Dselect-State", pkg->SelectedState);
@@ -496,6 +500,28 @@ bool aptitudeDepCache::build_selection_list(OpProgress &Prog, bool WithLock,
 		  if(!do_initselections)
 		    MarkKeep(pkg, false);
 		}
+
+	      // if the package is already in the version that we wanted to
+	      // target, but pkgstates still lists as "upgrade" with a candidate
+	      // version (downgrades always do, upgrades not always), mark as
+	      // dirty to update pkgstates, and reset "pkg_state".  otherwise:
+	      //
+	      // - if pkgstates is not written at most at the end of the current
+	      // session, the packages downgraded do not show up as "upgradable"
+	      // until there is an update forced for other reasons (see #787658
+	      // and #714429), even in subsequent invokations of aptitude
+	      //
+	      // - without resetting "pkg_state" it's marked for "upgrade" (or
+	      // "downgrade") again down in this function, and while for
+	      // "upgrades" it is not a problem, with "downgrades" it will not
+	      // show as upgradable in the current interactive session.
+	      std::string installed_ver = pkg.CurVersion() ? pkg.CurVersion() : "";
+	      if (pkg_state.upgrade && installed_ver == candver)
+		{
+		  pkg_state.upgrade = false;
+		  pkg_state.candver = "";
+		  dirty = true;
+		}
 	    }
 	  amt+=section.size();
 	  Prog.OverallProgress(amt, file_size, 1, _("Reading extended state information"));
@@ -549,15 +575,17 @@ bool aptitudeDepCache::build_selection_list(OpProgress &Prog, bool WithLock,
 	      MarkInstall(i, false);
 	    }
 	  else
-	    if(i.CurrentVer().end())
-	      MarkInstall(i, false);
-	    else
-	      {
-		SetReInstall(i, estate.reinstall);
+	    {
+	      if(i.CurrentVer().end())
+		MarkInstall(i, false);
+	      else
+		{
+		  SetReInstall(i, estate.reinstall);
 
-		if(estate.upgrade && state.Upgradable())
-		  MarkInstall(i, false);
-	      }
+		  if(estate.upgrade && state.Upgradable())
+		    MarkInstall(i, false);
+		}
+	    }
 	  break;
 	case pkgCache::State::Hold:
 	  if(!do_initselections)
@@ -735,6 +763,9 @@ bool aptitudeDepCache::save_selection_list(OpProgress &prog,
   if(status_fname == NULL)
     writeStateFile(&prog);
 
+  // helper class to save selection state of packages to dpkg database
+  aptitude::apt::dpkg::DpkgSelections dpkg_selections;
+
   string statefile=_config->FindDir("Dir::Aptitude::state", STATEDIR)+"pkgstates";
 
   FileFd newstate;
@@ -849,6 +880,32 @@ bool aptitudeDepCache::save_selection_list(OpProgress &prog,
 				      user_tags.c_str(),
 				      tailstr.c_str()));
 
+	    // dpkg-dselect state
+	    if (estate.original_selection_state != estate.selection_state
+		&& ! (estate.original_selection_state == pkgCache::State::Unknown && estate.selection_state == pkgCache::State::DeInstall) )
+	      {
+		// for internal reasons, apt's PkgIterator is always arch:any,
+		// not arch:all.  VerIterator can be arch:all, though.
+		std::string select_arch = i.Arch();
+
+		// all install, downgrade, upgrade, etc -- try to use candidate version
+		if (state.Install() && !GetCandidateVer(i).end())
+		  {
+		    select_arch = GetCandidateVer(i).Arch();
+		  }
+		// all delete, purge, hold, keep etc -- try to use current version
+		else if (! i.CurrentVer().end())
+		  {
+		    select_arch = i.CurrentVer().Arch();
+		  }
+
+		// add to selections to save
+		dpkg_selections.add(i.Name(), select_arch, estate.selection_state);
+
+		// new state is also the original now
+		estate.original_selection_state = estate.selection_state;
+	      }
+
 	    if(newstate.Failed() || !newstate.Write(line.c_str(), line.size()))
 	      {
 		_error->Error(_("Couldn't write state file"));
@@ -906,6 +963,26 @@ bool aptitudeDepCache::save_selection_list(OpProgress &prog,
 	      return false;
 	    }
 	}
+
+      // save selection state to dpkg database
+      //
+      // \todo Need to release lock before the operation, otherwise internal
+      // calls to dpkg fail with:
+      //
+      //   'error: dpkg status database is locked by another process'
+      //
+      // Review if dpkg provides a better way to do this, currently (Sep 2015)
+      // it does not.
+      apt_cache_file->ReleaseLock();
+      bool dpkg_selections_saved = dpkg_selections.save_selections();
+      if (! apt_cache_file->GainLock())
+	_error->Error(_("Could not regain the system lock!  (Perhaps another apt or dpkg is running?)"));
+      if (! dpkg_selections_saved)
+	{
+	  _error->Error(_("failed to save selections to dpkg database"));
+	  prog.Done();
+	  return false;
+	}
     }
 
   prog.Done();
@@ -1019,13 +1096,6 @@ void aptitudeDepCache::cleanup_after_change(undo_group *undo,
 	  // hard to track down bugs like #432411.
 	  pre_package_state_changed();
 
-	  int n=pkg->ID;
-	  n=n;
-	  char curM=PkgState[pkg->ID].Mode;
-	  curM=curM;
-	  char oldM=backup_state.PkgState[pkg->ID].Mode;
-	  oldM=oldM;
-
 	  if(alter_stickies &&
 	     PkgState[pkg->ID].Mode!=backup_state.PkgState[pkg->ID].Mode &&
 	     package_states[pkg->ID].selection_state==backup_state.AptitudeState[pkg->ID].selection_state)
@@ -1075,10 +1145,14 @@ void aptitudeDepCache::cleanup_after_change(undo_group *undo,
 	      PkgState[pkg->ID].Garbage != backup_state.PkgState[pkg->ID].Garbage ||
 	      package_states[pkg->ID].user_tags != backup_state.AptitudeState[pkg->ID].user_tags ||
 	      package_states[pkg->ID].new_package != backup_state.AptitudeState[pkg->ID].new_package)
-	visibly_changed = true;
+	{
+	  visibly_changed = true;
+	}
 
       if(visibly_changed && changed_packages != NULL)
-	changed_packages->insert(pkg);
+	{
+	  changed_packages->insert(pkg);
+	}
     }
 }
 
@@ -1157,6 +1231,15 @@ void aptitudeDepCache::internal_mark_delete(const PkgIterator &Pkg,
 					    bool Purge,
 					    bool unused_delete)
 {
+  // honour ::Purge-Unused in the main entry point for removing packages, it
+  // should catch cases of automatically installed and unused packages not
+  // purged (#724034 and others)
+  bool purge_unused = aptcfg->FindB(PACKAGE "::Purge-Unused", false);
+  if (unused_delete && purge_unused)
+    {
+      Purge = true;
+    }
+
   dirty=true;
 
   bool previously_to_delete=(*this)[Pkg].Delete();
@@ -1174,6 +1257,107 @@ void aptitudeDepCache::internal_mark_delete(const PkgIterator &Pkg,
       else
 	get_ext_state(Pkg).remove_reason=manual;
     }
+
+  // after marking a package for delete, check all the dependencies to see if
+  // they can be pro-actively marked for removal (because no other package
+  // depends on them).
+  //
+  // apt sometimes marks them as "Garbage", and so sweep() routines catch them
+  // later and they get removed within the same aptitude run, but sometimes it
+  // is not so, and they are only marked for removal in future sessions (see
+  // #637257, #478116 -- among many others).  so here is an attempt to directly
+  // mark for delete depends which seem safe to remove, because they are
+  // automatically installed and don't have other remaining rdeps.
+
+  if (!aptcfg->FindB(PACKAGE "::Delete-Unused", true))
+    return;
+
+  if (Pkg.CurrentVer().end())
+    return;
+
+  // from now and for the remaining of this function, these are "unused
+  // deletes", so set variable accordingly
+  unused_delete = true;
+
+  // not to purge unused lightly, can cause data loss -- see comments in #661188
+  Purge = aptcfg->FindB(PACKAGE "::Purge-Unused", false);
+
+  bool follow_recommends = aptcfg->FindB("APT::Install-Recommends", true) || aptcfg->FindB(PACKAGE "::Keep-Recommends", false);
+  bool follow_suggests   = aptcfg->FindB(PACKAGE "::Keep-Suggests", false) || aptcfg->FindB(PACKAGE "::Suggests-Important", false);
+
+  for (pkgCache::DepIterator dep = Pkg.CurrentVer().DependsList(); !dep.end(); ++dep)
+    {
+      // if not a valid target, not consider
+      auto dep_pkg = dep.TargetPkg();
+      if (dep_pkg.end())
+	{
+	  continue;
+	}
+
+      // consider only these type of dependencies
+      if (! ((dep->Type == pkgCache::Dep::Depends) ||
+	     (dep->Type == pkgCache::Dep::PreDepends) ||
+	     (dep->Type == pkgCache::Dep::Recommends && follow_recommends) ||
+	     (dep->Type == pkgCache::Dep::Suggests   && follow_suggests)))
+	{
+	  continue;
+	}
+
+      // special case for virtual packages
+      //
+      // if the considered dependency is a virtual package, look if both the
+      // virtual and provider packages can be removed (if they do not have other
+      // rdeps installed, etc)
+      if (is_virtual(dep_pkg))
+	{
+	  // consider all provided virtual packages
+	  for (pkgCache::PrvIterator dep_prv = dep_pkg.ProvidesList(); !dep_prv.end(); ++dep_prv)
+	    {
+	      // virtual package itself
+	      if (! can_remove_autoinstalled(dep_pkg, (*this), follow_recommends, follow_suggests)) {
+		continue;
+	      }
+
+	      // real package
+	      if (! can_remove_autoinstalled(dep_prv.OwnerPkg(), (*this), follow_recommends, follow_suggests)) {
+		continue;
+	      }
+
+	      // if we reach here, can delete the real package providing the
+	      // dependency
+	      internal_mark_delete(dep_prv.OwnerPkg(), Purge, unused_delete);
+	    }
+
+	  // it was a virtual package -- so stop processing the considered
+	  // dependency
+	  continue;
+	}
+
+      // if not virtual and no current version installed, no need to consider
+      auto dep_ver = dep_pkg.CurrentVer();
+      if (dep_ver.end())
+	{
+	  continue;
+	}
+
+      // consider only automatically installed dependencies, or dependencies
+      // which are not required on their own right
+      pkgDepCache::StateCache& state = (*this)[dep_pkg];
+      bool is_auto = is_auto_installed(state);
+      bool is_not_required = ! ((state.Flags & pkgCache::Flag::Essential) ||
+				(state.Flags & pkgCache::Flag::Important) ||
+				(dep_ver->Priority == pkgCache::State::Important) ||
+				(dep_ver->Priority == pkgCache::State::Required));
+
+      if (is_installed(dep_pkg) && is_auto && is_not_required)
+	{
+	  if (! can_remove_autoinstalled(dep_pkg, (*this), follow_recommends, follow_suggests)) {
+	    continue;
+	  }
+
+	  internal_mark_delete(dep_pkg, Purge, unused_delete);
+	}
+    }
 }
 
 void aptitudeDepCache::mark_keep(const PkgIterator &Pkg, bool Automatic, bool SetHold, undo_group *undo)
@@ -1357,12 +1541,17 @@ void aptitudeDepCache::mark_auto_installed(const PkgIterator &Pkg,
       return;
     }
 
-  action_group group(*this, undo);
+  // it is faster to check first
+  bool current_value = is_auto_installed((*this)[Pkg]);
+  if (set_auto != current_value)
+    {
+      action_group group(*this, undo);
 
-  pre_package_state_changed();
-  dirty=true;
+      pre_package_state_changed();
+      dirty=true;
 
-  MarkAuto(Pkg, set_auto);
+      MarkAuto(Pkg, set_auto);
+    }
 }
 
 // Undoers for the tag manipulators below.
@@ -1573,18 +1762,28 @@ void aptitudeDepCache::MarkFromDselect(const PkgIterator &Pkg)
 
   aptitude_state &state=get_ext_state(Pkg);
 
+  state.original_selection_state = static_cast<pkgCache::State::PkgSelectedState>(Pkg->SelectedState);
+
   if(Pkg->SelectedState!=state.selection_state)
     {
       switch(Pkg->SelectedState)
 	{
-	case pkgCache::State::Unknown:
-	  break;
 	case pkgCache::State::Purge:
 	  if( (!Pkg.CurrentVer().end()) || !((*this)[Pkg].iFlags&Purge) )
 	    mark_delete(Pkg, true, false, NULL);
 	  else
 	    mark_keep(Pkg, false, false, NULL);
 	  break;
+	case pkgCache::State::Unknown:
+	  // previously ignored (no action, just a break), it was causing
+	  // problems not respecting actions taken with other tools (see #328616
+	  // and duplicates).  so instead, if it's unknown to dpkg, mark it for
+	  // removal -- moved next to DeInstall and fall through.
+	  //
+	  // ('purge' is more dangerous, one could loose configuration files if
+	  // accidentally something goes wrong)
+	  //
+	  //break;
 	case pkgCache::State::DeInstall:
 	  if(!Pkg.CurrentVer().end())
 	    mark_delete(Pkg, false, false, NULL);
@@ -1813,14 +2012,22 @@ namespace
     for(pkgCache::DepIterator dep = maybeOrphan.RevDependsList();
 	!dep.end(); ++dep)
       {
-	if(dep.ParentPkg() != maybeOrphan &&
-	   (dep->Type == pkgCache::Dep::Depends ||
-	    dep->Type == pkgCache::Dep::PreDepends))
+	if ((dep.ParentPkg() != maybeOrphan) &&
+	    (dep->Type == pkgCache::Dep::Depends || dep->Type == pkgCache::Dep::PreDepends))
 	  {
+	    // only consider installed or to be installed packages
+	    const pkgDepCache::StateCache& state = cache[dep.ParentPkg()];
+	    if ( ! (state.Install() || (is_installed(dep.ParentPkg()) && ! state.Delete())))
+	      {
+		continue;
+	      }
+
 	    if(_system->VS->CheckDep(maybeOrphanCurrentVer.VerStr(),
 				     dep->CompareOp,
 				     dep.TargetVer()))
-	      trace_not_orphaned(maybeOrphan, reinstated, cache, not_orphaned);
+	      {
+		trace_not_orphaned(maybeOrphan, reinstated, cache, not_orphaned);
+	      }
 	  }
       }
 
@@ -1830,14 +2037,22 @@ namespace
       for(pkgCache::DepIterator dep = prv.ParentPkg().RevDependsList();
 	  !dep.end(); ++dep)
 	{
-	  if(dep.ParentPkg() != maybeOrphan &&
-	     (dep->Type == pkgCache::Dep::Depends ||
-	      dep->Type == pkgCache::Dep::PreDepends))
+	  if ((dep.ParentPkg() != maybeOrphan) &&
+	      (dep->Type == pkgCache::Dep::Depends || dep->Type == pkgCache::Dep::PreDepends))
 	    {
+	      // only consider installed or to be installed packages
+	      const pkgDepCache::StateCache& state = cache[dep.ParentPkg()];
+	      if ( ! (state.Install() || (is_installed(dep.ParentPkg()) && ! state.Delete())))
+		{
+		  continue;
+		}
+
 	      if(_system->VS->CheckDep(prv.ProvideVersion(),
 				       dep->CompareOp,
 				       dep.TargetVer()))
-		trace_not_orphaned(maybeOrphan, reinstated, cache, not_orphaned);
+		{
+		  trace_not_orphaned(maybeOrphan, reinstated, cache, not_orphaned);
+		}
 	    }
 	}
   }
@@ -1928,7 +2143,6 @@ void aptitudeDepCache::sweep()
 	      LOG_DEBUG(logger, "aptitudeDepCache::sweep(): provisionally scheduling "
 			<< pkg.FullName(false) << " for reinstatement.");
 
-
 	      reinstated.insert(pkg);
 	    }
 	}
@@ -2239,7 +2453,7 @@ bool aptitudeDepCache::is_held(const PkgIterator &pkg)
 bool aptitudeDepCache::MarkFollowsRecommends()
 {
   return pkgDepCache::MarkFollowsRecommends() ||
-    aptcfg->FindB("Apt::Install-Recommends", true) ||
+    aptcfg->FindB("APT::Install-Recommends", true) ||
     aptcfg->FindB(PACKAGE "::Keep-Recommends", false);
 }
 
