@@ -37,14 +37,14 @@
 
 #include <xapian.h>
 
-#include <algorithm>
-
-#include <boost/scoped_ptr.hpp>
 #include <boost/unordered_map.hpp>
 
 #include "serialize.h"
 #include "../config_signal.h"
 
+#include <algorithm>
+#include <memory>
+
 using aptitude::util::progress_info;
 using boost::unordered_map;
 using cwidget::util::transcode;
@@ -173,7 +173,7 @@ namespace aptitude
 	 *  matched by this pattern.
 	 */
 	bool maybe_contains_package(const pkgCache::PkgIterator &pkg,
-				    const boost::scoped_ptr<debtags_db> &db) const
+				    const std::unique_ptr<debtags_db> &db) const
 	{
 	  if(!matched_packages_valid || !db)
 	    return true;
@@ -306,7 +306,7 @@ namespace aptitude
 
       // Either a pointer to the debtags database, or NULL if it
       // couldn't be initialized.
-      boost::scoped_ptr<debtags_db> db;
+      std::unique_ptr<debtags_db> db;
 
       // Maps "top-level" patterns to their Xapian information (that
       // is, the corresponding query and/or query results).  Term hit
@@ -392,7 +392,7 @@ namespace aptitude
 	  }
       }
 
-      const boost::scoped_ptr<debtags_db> &get_db() const
+      const std::unique_ptr<debtags_db> &get_db() const
       {
 	return db;
       }
@@ -1492,18 +1492,9 @@ namespace aptitude
 		  }
 	      }
 
-	    {
-	      pkgCache::PkgIterator pkg(target.get_package_iterator(cache));
-	      const char *pkg_section = pkg.Section();
+	    // return null as last resort
+	    return NULL;
 
-	      if(pkg_section != NULL)
-		return evaluate_regexp(p,
-				       p->get_section_regex_info(),
-				       pkg_section,
-				       debug);
-	      else
-		return NULL;
-	    }
 	    break;
 
 	  case pattern::source_package:
@@ -1511,6 +1502,24 @@ namespace aptitude
 	      if(!target.get_has_version())
 		return NULL;
 
+#if APT_PKG_MAJOR >= 5
+	      // with apt-1.1:
+	      //
+	      // - SourcePkg (and Version) are in the binary cache and available via
+	      //   the VerIterator; much faster than parsing the pkgRecord
+	      //
+	      // - defaults to package name, no need to check if it's empty
+	      pkgCache::VerIterator ver(target.get_version_iterator(cache));
+	      if ( ! ver.end() )
+		{
+		  ref_ptr<match> rval = evaluate_regexp(p,
+							p->get_source_package_regex_info(),
+							ver.SourcePkgName(),
+							debug);
+		  if (rval.valid())
+		    return rval;
+		}
+#else
 	      bool checked_real_package = false;
 
 	      pkgCache::PkgIterator pkg(target.get_package_iterator(cache));
@@ -1547,6 +1556,7 @@ namespace aptitude
 			return rval;
 		    }
 		}
+#endif
 
 	      return NULL;
 	    }
@@ -1557,9 +1567,27 @@ namespace aptitude
 	      if(!target.get_has_version())
 		return NULL;
 
+#if APT_PKG_MAJOR >= 5
+	      // with apt-1.1:
+	      //
+	      // - SourcePkg (and Version) are in the binary cache and available via
+	      //   the VerIterator; much faster than parsing the pkgRecord
+	      //
+	      // - defaults to package name, no need to check if it's empty
+	      pkgCache::VerIterator ver(target.get_version_iterator(cache));
+	      if ( ! ver.end() )
+		{
+		  ref_ptr<match> rval = evaluate_regexp(p,
+							p->get_source_version_regex_info(),
+							ver.SourceVerStr(),
+							debug);
+		  if (rval.valid())
+		    return rval;
+		}
+#else
 	      bool checked_real_package = false;
 
-	      pkgCache::PkgIterator pkg(target.get_package_iterator(cache));
+	      //pkgCache::PkgIterator pkg(target.get_package_iterator(cache));
 	      pkgCache::VerIterator ver(target.get_version_iterator(cache));
 
 	      for(pkgCache::VerFileIterator vf = ver.FileList();
@@ -1593,6 +1621,7 @@ namespace aptitude
 			return rval;
 		    }
 		}
+#endif
 
 	      return NULL;
 	    }
