@@ -591,6 +591,47 @@ void handle_message_logged(const char *sourceFilename,
     }
 }
 
+
+/** Signal handler
+ *
+ * Clean up routines when we have to close down abruptly, to not leave files
+ * behind, etc. -- e.g. see: https://bugs.debian.org/558352
+ */
+void signal_handler(int signum)
+{
+  //fprintf(stderr, "Top-level signal handler: %d\n", signum);
+
+  // remove directory in /tmp
+  temp::shutdown();
+
+  // re-raise signal with default handling, to set correct exit status and so on
+  signal(signum, SIG_DFL);
+  raise(signum);
+}
+
+/** Install signal handlers
+ *
+ * Register which process signals do we want to handle, and with which handlers
+ */
+void install_signal_handlers()
+{
+  // termination signals
+  signal(SIGTERM, signal_handler);
+  signal(SIGINT,  signal_handler);
+  signal(SIGHUP,  signal_handler);
+
+  // for debugging purposes like core dumps, should not delete intermediate
+  // files, so we leave the default
+  //
+  //signal(SIGQUIT, signal_handler);
+
+  // abrupt termination
+  signal(SIGILL,  signal_handler);
+  signal(SIGSEGV, signal_handler);
+  signal(SIGBUS,  signal_handler);
+  signal(SIGABRT, signal_handler);
+}
+
 int main(int argc, char *argv[])
 {
   // Block signals that we want to sigwait() on by default and put the
@@ -614,6 +655,9 @@ int main(int argc, char *argv[])
     pthread_sigmask(SIG_BLOCK, &mask, NULL);
   }
 
+  // handle signals, like abrupt termination (do clean-up, etc)
+  install_signal_handlers();
+
   srandom(time(0));
 
   // See earlier note
@@ -649,7 +693,7 @@ int main(int argc, char *argv[])
   bool simulate = aptcfg->FindB(PACKAGE "::CmdLine::Simulate", false) ||
     aptcfg->FindB(PACKAGE "::Simulate", false);
   bool download_only=aptcfg->FindB(PACKAGE "::CmdLine::Download-Only", false);;
-  bool arch_only = aptcfg->FindB("Apt::Get::Arch-Only", false);
+  bool arch_only = aptcfg->FindB("APT::Get::Arch-Only", false);
 
   bool update_only=false, install_only=false, queue_only=false;
   bool autoclean_only = false;
@@ -777,11 +821,11 @@ int main(int argc, char *argv[])
 	  seen_quiet = true;
 	  break;
 	case 'r':
-	  aptcfg->SetNoUser("Apt::Install-Recommends", "true");
+	  aptcfg->SetNoUser("APT::Install-Recommends", "true");
 	  break;
 	case 'R':
-	  aptcfg->SetNoUser("Apt::Install-Recommends", "false");
-	  aptcfg->SetNoUser("Apt::AutoRemove::RecommendsImportant", "true");
+	  aptcfg->SetNoUser("APT::Install-Recommends", "false");
+	  aptcfg->SetNoUser("APT::AutoRemove::RecommendsImportant", "true");
 	  break;
 	case 't':
 	  aptcfg->SetNoUser("APT::Default-Release", optarg);
