@@ -3,6 +3,7 @@
 //  The pkg_columnizer class.
 //
 //  Copyright 1999-2005, 2007-2008, 2010 Daniel Burrows
+//  Copyright 2012-2015 Manuel A. Fernandez Montecelo
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -161,7 +162,7 @@ cw::column_disposition pkg_item::pkg_columnizer::setup_column(const pkgCache::Pk
 	  pkgCache::VerIterator inst_ver=(*apt_cache_file)[pkg].InstVerIter(*apt_cache_file);
 
 	  // TODO: move this to a generic file (common with cmdline.cc)
-	  int dsize=(inst_ver.end()?0:inst_ver->InstalledSize)
+	  long long dsize=(inst_ver.end()?0:inst_ver->InstalledSize)
 	    -(pkg.CurrentVer().end()?0:pkg.CurrentVer()->InstalledSize);
 
 	  if(dsize==0)
@@ -220,12 +221,10 @@ cw::column_disposition pkg_item::pkg_columnizer::setup_column(const pkgCache::Pk
 	    return cw::column_disposition("c", 0);
 	  case pkgCache::State::Installed:
 	    return cw::column_disposition("i", 0);
-#ifdef APT_HAS_TRIGGERS
 	  case pkgCache::State::TriggersAwaited:
 	    return cw::column_disposition("W", 0);
 	  case pkgCache::State::TriggersPending:
 	    return cw::column_disposition("T", 0);
-#endif
 	  default:
 	    return cw::column_disposition("E", 0);
 	  }
@@ -256,12 +255,10 @@ cw::column_disposition pkg_item::pkg_columnizer::setup_column(const pkgCache::Pk
 	    return cw::column_disposition(_("config-files"), 0);
 	  case pkgCache::State::Installed:
 	    return cw::column_disposition(_("installed"), 0);
-#ifdef APT_HAS_TRIGGERS
 	  case pkgCache::State::TriggersAwaited:
 	    return cw::column_disposition(_("triggers-awaited"), 0);
 	  case pkgCache::State::TriggersPending:
 	    return cw::column_disposition(_("triggers-pending"), 0);
-#endif
 	  default:
 	    return cw::column_disposition(_("ERROR"), 0);
 	  }
@@ -668,16 +665,25 @@ void pkg_item::pkg_columnizer::setup_columns(bool force_update)
     delete columns;
   if(force_update || !columns)
     {
-      std::wstring cfg;
+      std::string cfg_option_name = PACKAGE "::UI::Package-Display-Format";
+      std::string aptcfg_display_format = aptcfg->Find(cfg_option_name.c_str(), default_pkgdisplay);
+      bool format_ok = check_valid_display_format(aptcfg_display_format, cfg_option_name);
+      if (!format_ok)
+	{
+	  aptcfg_display_format = default_pkgdisplay;
+	  _error->Warning(_("Using default for %s string: '%s'"), cfg_option_name.c_str(), aptcfg_display_format.c_str());
+	}
 
-      if(!cw::util::transcode(aptcfg->Find(PACKAGE "::UI::Package-Display-Format",
-				 default_pkgdisplay).c_str(),
-		    cfg))
+      std::wstring cfg;
+      if(!cw::util::transcode(aptcfg_display_format.c_str(), cfg))
 	_error->Errno("iconv", _("Unable to transcode package display format after \"%ls\""), cfg.c_str());
       else
-	columns=parse_columns(cfg,
-			      pkg_columnizer::parse_column_type,
-			      pkg_columnizer::defaults);
+	{
+	  columns=parse_columns(cfg,
+				pkg_columnizer::parse_column_type,
+				pkg_columnizer::defaults);
+	}
+
       if(!columns)
 	{
 	  cfg.clear();
@@ -708,3 +714,19 @@ std::wstring pkg_item::pkg_columnizer::format_column_names(unsigned int width)
   cw::config::empty_column_parameters p;
   return pkg_genheaders(*columns).layout_columns(width, p);
 }
+
+bool pkg_item::pkg_columnizer::check_valid_display_format(const std::string& display_format, const std::string& cfg_option_name)
+{
+  for (size_t i = 0; i < display_format.size(); ++i)
+    {
+      auto it = display_format[i];
+      if (iscntrl(it) || it == '\n' || it == '\\')
+	{
+	  _error->Error(_("Forbidden character in position %zu of %s string: '%s'"),
+			i, cfg_option_name.c_str(), display_format.c_str());
+	  return false;
+	}
+    }
+
+  return true;
+}
